/*
 * $Id: hmgapp.prg 664 2011-07-11 12:56:04Z concentra $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hbclass.ch"
#include "common.ch"
#include "hmg_qtlocale.ch"

/*----------------------------------------------------------------------*/

CLASS HMGAPP FROM GLOBSHARED

   DATA cClass                         INIT   "HMGAPP"

// data: please preserve alphabetic order.
   DATA oQtObject                      INIT   NIL     // to have compatibility with Hmg4 object parent system
   DATA aControls                      INIT   {}      // to have compatibility with Hmg4 object parent system

// method: please preserve alphabetic order. Methods "New" and "Create" must be the first if present
   METHOD New
   METHOD AddData                                     // to have compatibility with Hmg4 object parent system
   METHOD Country
   METHOD CurrHmgWin
   METHOD CurrQtWin
   METHOD DelData
   METHOD Language
   METHOD Localized                    SETGET
   METHOD OnStartUp                    SETGET
   METHOD OnShutDown                   SETGET
   METHOD PlugInDir                    SETGET
   METHOD Quit
   METHOD Reset
   METHOD RegisterResource
   METHOD Resources                    SETGET
   METHOD Start
   METHOD UnRegisterResource

ENDCLASS

/*----------------------------------------------------------------------*/

METHOD New() CLASS HMGAPP

   // check if it's already started.
   IF ::s_lQtStarted == .T.
      RETURN NIL
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD AddData( cName , oObject ) CLASS HMGAPP

   __objAddData( Self , LOWER( cName ) )
   ::&cName := oObject

RETURN Self

/*----------------------------------------------------------------------*/

METHOD Country() CLASS HMGAPP
RETURN ::s_oQtLocale:Country()

/*----------------------------------------------------------------------*/

METHOD CurrHmgWin() CLASS HMGAPP
RETURN ::s_oCurWindow

/*----------------------------------------------------------------------*/

METHOD CurrQtWin() CLASS HMGAPP
RETURN ::s_oQtApp:activeWindow()

/*----------------------------------------------------------------------*/

METHOD DelData( cName ) CLASS HMGAPP

   IF __objHasMsg( Self, LOWER( cName ) )      // This tells that cName exists in the object.
      ::&cName := NIL
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Language() CLASS HMGAPP
RETURN ::s_oQtLocale:Language()

/*----------------------------------------------------------------------*/

METHOD Localized( nLanguage, nCountry ) CLASS HMGAPP

   IF PCOUNT() == 0
      RETURN { ::s_nLanguage, ::s_nCountry }     // attention: an array it's returned
   ELSEIF ::s_lQtStarted == .F.
      IF hb_IsNumeric( nLanguage ) == .T.
         ::s_nLanguage := nLanguage
      ENDIF
      IF hb_IsNumeric( nCountry ) == .T.
         ::s_nCountry := nCountry
      ENDIF
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD OnStartUp( bOnStartUp ) CLASS HMGAPP

   IF PCOUNT() == 0
      RETURN ::s_bOnStartUp
   ELSEIF ::s_lQtStarted == .F. .AND. hb_IsBlock( bOnStartUp ) == .T.
      ::s_bOnStartUp := bOnStartUp
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD OnShutDown( bOnShutDown ) CLASS HMGAPP

   IF PCOUNT() == 0
      RETURN ::s_bOnShutDown
   ELSEIF ::s_lQtStarted == .F. .AND. hb_IsBlock( bOnShutDown ) == .T.
      ::s_bOnShutDown := bOnShutDown
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD PlugInDir( cPlugInDir ) CLASS HMGAPP

   IF PCOUNT() == 0
      RETURN ::s_cPlugInDir
   ELSEIF ::s_lQtStarted == .F. .AND.  hb_IsChar( cPlugInDir ) == .T. .AND. hb_DirExists( ALLTRIM( cPlugInDir ) ) ==.T.
      ::s_cPlugInDir := ALLTRIM( cPlugInDir )
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD Quit() CLASS HMGAPP

   // run s_bOnShutDown
   IF hb_IsBlock( ::s_bOnShutDown ) == .T. .AND. ::s_lQtStarted == .T.
      EVAL( ::s_bOnShutDown )
   ENDIF

   IF ::s_lQtStarted == .T.
//????      ::s_oQtApp:CloseAllWindows()
      Self:UnRegisterResource()
      ::s_oQtApp:Exit()
      ::s_oQtApp:Quit()
   ENDIF

   Self:Reset()

   hb_gcall()

RETURN Self

/*----------------------------------------------------------------------*/

/*
 * ATTENTION: this method need QApplications started.
 * For this reason you must prefer use this after Application it's already started
 */
METHOD RegisterResource( cResource ) CLASS HMGAPP

   LOCAL cResFileName

   // check if it's already started
   IF ::s_lQtStarted == .F.
      HMGAPP():Start()
   ENDIF

   IF PCOUNT() == 1 .AND. VALTYPE( cResource ) == "C"
      cResFileName := "HBQTRES_" + ALLTRIM( cResource ) + "()"
      IF TYPE( cResFileName ) == "UI"  // to find if exist this function
         IF ::s_oQtRes:registerResource_1( &cResFileName ) == .T.
            AADD( ::s_aResources, cResFileName )
         ENDIF
      ENDIF
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD Reset() CLASS HMGAPP

   ::s_oQtApp           := NIL
   ::s_oQtRes           := NIL
   ::s_aResources       := {}
   ::s_oDefaultParent   := NIL
   ::s_nObjectCounter   := 0
   ::s_oMainWindow      := NIL
   ::s_lGridCellFlag    := .F.
   ::s_oMainMenu        := NIL
   ::s_oParentWindow    := NIL
   ::s_cNewPopUpMenuName:= NIL
   ::s_cNewMenuItemName := NIL

   ::s_nCountry         := NIL
   ::s_oCurWindow       := NIL
   ::s_nLanguage        := NIL
   ::s_bOnShutDown      := NIL
   ::s_bOnStartUp       := NIL
   ::s_cPlugInDir       := NIL
   ::s_oQtLocale        := NIL
   ::s_lQtStarted       := .F.
   ::s_oTopParent       := NIL

RETURN Self

/*----------------------------------------------------------------------*/

/*
 * ATTENTION: this method no need QApplications started, but can be used ONLY before start
 * xResource can be: a FileName (ie "ResourceFileName") or an Array (ie {"ResFile1", "ResFile2"})
 */
METHOD Resources( xResource ) CLASS HMGAPP

   LOCAL cResFileName, nX

   IF PCOUNT() == 0
      RETURN ::s_aResources
   ELSEIF ::s_lQtStarted == .F. .AND. hb_IsChar( xResource ) == .T.
      cResFileName := "HBQTRES_" + ALLTRIM( xResource ) + "()"
      IF TYPE( cResFileName ) == "UI"  // to find if exist this function
         AADD( ::s_aResources, cResFileName )
      ENDIF
   ELSEIF ::s_lQtStarted == .F. .AND. hb_IsArray( xResource ) == .T.
      FOR nX := 1 TO LEN( xResource )
         cResFileName := "HBQTRES_" + ALLTRIM( xResource[nX] ) + "()"
         IF TYPE( cResFileName ) == "UI"  // to find if exist this function
            AADD( ::s_aResources, cResFileName )
         ENDIF
      NEXT nX
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD Start() CLASS HMGAPP

   LOCAL cPath, cFile, cExt, nX, cResFileName

   // check if it's already started
   IF ::s_lQtStarted == .T.
      RETURN Self
   ENDIF

   // split program name for two reason
   // 1. to have the current path
   // 2. to have the program name and register default resource with program name
   hb_fNameSplit( hb_ProgName(), @cPath, @cFile, @cExt )

   ::s_lQtStarted := .T.
   ::s_oQtApp     := QApplication()
   ::s_oQtRes     := QResource()

   // localization
   IF VALTYPE( ::s_nLanguage ) == "U"
      ::s_oQtLocale     := QLocale()
      ::s_oQtLocale:setDefault( QLocale() )
   ELSEIF VALTYPE( ::s_nLanguage ) == "N"
      IF VALTYPE( ::s_nCountry ) == "U" ; ::s_nCountry := 0 ; ENDIF     // 0 is like QLocale_AnyCountry
      ::s_oQtLocale     := QLocale( ::s_nLanguage, ::s_nCountry )
      ::s_oQtLocale:setDefault( QLocale( ::s_nLanguage, ::s_nCountry ) )
   ENDIF

   // if ::s_cPlugInDir not evaluated create a default <exePath\Qt\plugins>. If qt.conf exists take that
   IF VALTYPE( ::s_cPlugInDir ) == "U"
      ::s_cPlugInDir := cPath + "Qt" + hb_ps() + "plugins"
   ENDIF
   ::s_oQtApp:addLibraryPath( ::s_cPlugInDir )

   ::s_oDefaultParent   := Self
   ::s_oTopParent       := Self
   ::oQtObject          := ::s_oQtApp:DeskTop()

   // by default register resource file with the same name of program (see harbour)
   Self:RegisterResource( cFile )
   FOR nX := 1 TO LEN( ::s_aResources )
      cResFileName := ::s_aResources[nX]
      IF TYPE( cResFileName ) == "UI"  // to find if exist this function
         ::s_oQtRes:registerResource_1( &cResFileName )
      ENDIF
   NEXT nX

   //    check OnStartUp
   IF hb_IsBlock( ::s_bOnStartUp ) == .T.
      EVAL( ::s_bOnStartUp )
   ENDIF

RETURN Self

/*----------------------------------------------------------------------*/

METHOD UnRegisterResource() CLASS HMGAPP

   LOCAL nI, cResFileName

   FOR nI := 1 TO LEN( ::s_aResources )
      cResFileName := ::s_aResources[nI]
      ::s_oQtRes:unregisterResource_1( &cResFileName )
   NEXT nI

   ::s_aResources := {}

RETURN Self
