/*
 * $Id: virtualwindow.prg 659 2011-07-06 17:37:29Z concentra $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hbclass.ch"
#include "common.ch"
#include "hbqtgui.ch"
#include "hmg.ch"

/*----------------------------------------------------------------------*/

CLASS VIRTUALWINDOW FROM BASIC

   // Internal Data
   DATA cClass                                    INIT   'VIRTUALWINDOW'
   DATA aKeyBoardEvents                           INIT   {}
   DATA bOnHscrollBox                             INIT   NIL
   DATA bOnInit                                   INIT   NIL
   DATA bOnInteractiveClose                       INIT   NIL
   DATA bOnMaximize                               INIT   NIL
   DATA bOnMinimize                               INIT   NIL
   DATA bOnPaint                                  INIT   NIL
   DATA bOnScrollLeft                             INIT   NIL
   DATA bOnScrollRight                            INIT   NIL
   DATA bOnScrollUp                               INIT   NIL
   DATA bOnScrollDown                             INIT   NIL
   DATA bOnSize                                   INIT   NIL
   DATA bOnVscrollBox                             INIT   NIL

   // Properties

   // Events
   METHOD AddWindowFocus
   METHOD OnGotFocus                              SETGET
   METHOD OnHScrollBox                            SETGET
   METHOD OnInit                                  SETGET
   METHOD OnInteractiveClose                      SETGET
   METHOD OnLostFocus                             SETGET
   METHOD OnMaximize                              SETGET
   METHOD OnMinimize                              SETGET
   METHOD OnPaint                                 SETGET
   METHOD OnScrollUp                              SETGET
   METHOD OnScrollDown                            SETGET
   METHOD OnScrollLeft                            SETGET
   METHOD OnScrollRight                           SETGET
   METHOD OnSize                                  SETGET
   METHOD OnStateChange                           SETGET
   METHOD OnVScrollBox                            SETGET
   METHOD WindowStateChanged                      SETGET

   // Methods
   METHOD Center
   METHOD Capture
   METHOD Print
   METHOD Maximize
   METHOD Minimize

   // Keyboard Events Methods
   METHOD DoKeyBoardEvents
   METHOD RemoveKeyBoardEvent                     SETGET
   METHOD StoreKeyBoardEvent                      SETGET

   // Region Methods
   METHOD SetRegion                               SETGET
   METHOD ResetRegion                             SETGET
   METHOD Release

ENDCLASS

/*----------------------------------------------------------------------*/
//                              Properties
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
//                               Events
/*----------------------------------------------------------------------*/

METHOD AddWindowFocus() CLASS VIRTUALWINDOW

   LOCAL cObjName := ::Name

   ::s_oCurWindow  := ::oParent:&cObjName

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnGotFocus( bValue ) CLASS VIRTUALWINDOW

   IF Pcount() == 0
      RETURN ::bOnGotFocus
   ELSEIF Pcount() == 1
      IF ::oQTObject:focusPolicy() == Qt_NoFocus
         ::oQTObject:setFocusPolicy( Qt_StrongFocus )
      ENDIF
      ::bOnGotFocus := bValue
      IF .NOT. ::oQTObject:connect( QEvent_FocusIn , {||Eval( bValue ), Self:AddWindowFocus() } )
         ::oQTObject:disconnect( QEvent_FocusIn )
         IF .NOT. ::oQTObject:connect( QEvent_FocusIn , {||Eval( bValue ), Self:AddWindowFocus() } )
            // Error
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnHScrollBox( bValue ) CLASS VIRTUALWINDOW

   IF pcount() == 0
      RETURN ::bOnHScrollBox
   ELSEIF pcount() == 1
      ::bOnHScrollBox := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnInit( bValue ) CLASS VIRTUALWINDOW

   IF Pcount() == 0
      RETURN ::bOnInit
   ELSEIF Pcount() == 1
      ::bOnInit := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnInteractiveClose( bValue ) CLASS VIRTUALWINDOW

   IF pcount() == 0
      RETURN ::bOnInteractiveClose
   ELSEIF pcount() == 1
      ::bOnInteractiveClose := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnLostFocus( bValue ) CLASS VIRTUALWINDOW

   IF Pcount() == 0
      RETURN ::bOnLostFocus
   ELSEIF Pcount() == 1
      IF ::oQTObject:focusPolicy() == Qt_NoFocus
         ::oQTObject:setFocusPolicy( Qt_StrongFocus )
      ENDIF
      ::bOnLostFocus := bValue
      IF .NOT. ::oQTObject:connect( QEvent_FocusOut , {||Eval( bValue )} )
         ::oQTObject:disconnect( QEvent_FocusOut )
         IF .NOT. ::oQTObject:connect( QEvent_FocusOut , {||Eval( bValue )} )
             // Error
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnMaximize( bValue ) CLASS VIRTUALWINDOW

   IF Pcount() == 0
      RETURN ::bOnMaximize
   ELSEIF Pcount() == 1
      ::bOnMaximize := bValue
      Self:OnStateChange()
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnMinimize( bValue ) CLASS VIRTUALWINDOW

   IF Pcount() == 0
      RETURN ::bOnMinimize
   ELSEIF Pcount() == 1
      ::bOnMinimize := bValue
      Self:OnStateChange()
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnPaint( bValue ) CLASS VIRTUALWINDOW

   IF Pcount() == 0
      RETURN ::bOnPaint
   ELSEIF Pcount() == 1
      ::bOnPaint := bValue
      * Paint event sub-classing for Onpaint event
      IF .NOT. ::oQTObject:connect( QEvent_Paint , {||( Eval( bValue ) )} )
         ::oQTObject:disconnect( QEvent_Paint )
         IF .NOT. ::oQTObject:connect( QEvent_Paint , {||( Eval( bValue ) )} )
            // Error
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnScrollUp( bValue ) CLASS VIRTUALWINDOW

   IF pcount() == 0
      RETURN ::bOnScrollUp
   ELSEIF pcount() == 1
      ::bOnScrollUp := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnScrollDown( bValue ) CLASS VIRTUALWINDOW

   IF pcount() == 0
      RETURN ::bOnScrollDown
   ELSEIF pcount() == 1
      ::bOnScrollDown := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnScrollLeft( bValue ) CLASS VIRTUALWINDOW

   IF pcount() == 0
      RETURN ::bOnScrollLeft
   ELSEIF pcount() == 1
      ::bOnScrollLeft := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnScrollRight( bValue ) CLASS VIRTUALWINDOW

   IF pcount() == 0
      RETURN ::bOnScrollRight
   ELSEIF pcount() == 1
      ::bOnScrollRight := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnSize( bValue ) CLASS VIRTUALWINDOW

   IF Pcount() == 0
      RETURN ::bOnSize
   ELSEIF Pcount() == 1
      ::bOnSize := bValue
      IF .NOT. ::oQTObject:connect( QEvent_Resize , { || Eval( ::bOnSize ) } )
         ::oQTObject:disconnect( QEvent_Resize )
         IF .NOT. ::oQTObject:connect( QEvent_Resize , { || Eval( ::bOnSize ) } )
            // Error
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnStateChange() CLASS VIRTUALWINDOW

   IF .NOT. ::oQTObject:connect( QEvent_WindowStateChange , { | e | ::WindowStateChanged( e ) } )
      ::oQTObject:disconnect( QEvent_WindowStateChange )
      IF .NOT. ::oQTObject:connect( QEvent_WindowStateChange , { | e | ::WindowStateChanged( e ) } )
         // Error
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnVScrollBox( bValue ) CLASS VIRTUALWINDOW

   IF pcount() == 0
      RETURN ::bOnVScrollBox
   ELSEIF pcount() == 1
      ::bOnVScrollBox := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD WindowStateChanged( e ) CLASS VIRTUALWINDOW
//   LOCAL oWindowStateChangeEvent := QWindowStateChangeEvent():from( e )
   LOCAL nState := ::oQTObject:windowState()

   HB_SYMBOL_UNUSED( e )

   IF nState == Qt_WindowMinimized
      IF ValType( ::bOnMinimize ) != 'U'
         Eval( ::bOnMinimize )
      ENDIF
   ELSEIF nState == Qt_WindowMaximized
      IF ValType( ::bOnMaximize ) != 'U'
         Eval( ::bOnMaximize )
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD DoKeyBoardEvents( e ) CLASS VIRTUALWINDOW
   LOCAL oKeyEvent := e

   LOCAL nKey := oKeyEvent:key()
   LOCAL nModifiers := oKeyEvent:modifiers()
   LOCAL i, lReturnValue

   IF Len( ::aKeyBoardEvents ) == 0
      RETURN .F.  && Nothing to process!
   ENDIF
   i := aScan( ::aKeyBoardEvents,{|x| x[1] == nKey .and. x[2] == nModifiers} )
   IF i > 0
      lReturnValue := Eval( ::aKeyBoardEvents[i,3] )
      IF ValType( lReturnValue ) == 'L'
         RETURN lReturnValue
      ENDIF
   ENDIF

   RETURN .F.

/*----------------------------------------------------------------------*/

METHOD RemoveKeyBoardEvent( nKey,nModifiers ) CLASS VIRTUALWINDOW
   LOCAL i

   IF Len( ::aKeyBoardEvents ) == 0
      RETURN NIL  && Nothing to process!
   ENDIF
   i := aScan( ::aKeyBoardEvents,{|x| x[1] == nKey .and. x[2] == nModifiers} )
   IF i > 0
      aDel( ::aKeyBoardEvents, i )
      aSize( ::aKeyBoardEvents,len( ::aKeyBoardEvents ) - 1 )
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD StoreKeyBoardEvent( nKey,nModifiers ) CLASS VIRTUALWINDOW
   LOCAL i

   IF Len( ::aKeyBoardEvents ) == 0
      RETURN NIL  // Nothing to process!
   ENDIF
   i := aScan( ::aKeyBoardEvents,{|x| x[1] == nKey .and. x[2] == nModifiers} )
   IF i > 0
      RETURN ::aKeyBoardEvents [i] [3]
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/
//                                 Methods
/*----------------------------------------------------------------------*/

METHOD Capture( cPathFileOut, nRow, nCol, nWidth, nHeight ) CLASS VIRTUALWINDOW

   LOCAL oPixmap
   LOCAL oQPixTmp := QPixmap()
   LOCAL cPath, cFile, cExt

   IF hb_IsNil( cPathFileOut )
      cPathFileOut := hb_ValToStr( ::Name() )
   ENDIF
   IF hb_IsNil( nRow )
      nRow := 0
   ENDIF
   IF hb_IsNil( nCol )
      nCol := 0
   ENDIF
   IF hb_IsNil( nWidth )
      nWidth := ::Width()
   ENDIF
   IF hb_IsNil( nHeight )
      nHeight := ::Height()
   ENDIF

   hb_FNameSplit( cPathFileOut, @cPath, @cFile, @cExt )
   cExt := StrTran( cExt, ".", "" )

   IF Empty( cExt ) .Or. ! ( cExt $ "pngbmp" )
      cExt := "png"
      cPathFileOut := cPath + cFile + "." + cExt
   ENDIF

   oPixmap := oQPixTmp:grabWidget( ::oQTObject, nCol, nRow, nWidth, nHeight )

   RETURN oPixmap:save( cPathFileOut, cExt, -1 )

/*----------------------------------------------------------------------*/

METHOD Center CLASS VIRTUALWINDOW

   ::oQTObject:Move( Int( ( GetDesktopWidth() - ::oQTObject:Width ) / 2 ) , Int( ( GetDesktopHeight() - ::oQTObject:Height ) / 2 ) )

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Maximize CLASS VIRTUALWINDOW

   ::oQTObject:showMaximized()

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Minimize CLASS VIRTUALWINDOW

   ::oQTObject:showMinimized()

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Print( lPreview , ldialog , nRow , nCol , nWidth , nHeight ) CLASS VIRTUALWINDOW

   RETURN { lPreview , ldialog , nRow , nCol , nWidth , nHeight }

/*----------------------------------------------------------------------*/
//                             Region  Methods
/*----------------------------------------------------------------------*/

METHOD SetRegion( aOptions ) CLASS VIRTUALWINDOW
   LOCAL oRegion
*   LOCAL oPolygon, aPoints, i

   IF PCount() == 1
      DO CASE
      CASE aOptions[ 1 ] == 'RECTANGULAR'
         oRegion := QRegion( aOptions[ 3 ], aOptions[ 2 ], aOptions[ 4 ], aOptions[ 5 ], 0 )
         ::oQTObject:setMask( oRegion )
      CASE aOptions[ 1 ] == 'ELLIPTIC'
         oRegion := QRegion( aOptions[ 3 ], aOptions[ 2 ], aOptions[ 4 ], aOptions[ 5 ], 1 )
         ::oQTObject:setMask( oRegion )
      CASE aOptions[ 1 ] == 'POLYGONAL' // Not Working!
*          oPolygon := QPolygon( Len( aOptions[ 2 ] ) )
*          aPoints := aOptions[ 2 ]
*          FOR i := 1 TO Len( aOptions[ 2 ] )
*             oPolygon:setPoint( i, aPoints[ i, 2 ], aPoints[ i, 1 ] )
*          NEXT i
*          IF aOptions[ 3 ] // Alternate Fill
*             oRegion := QRegion( oPolygon, 0 )
*          ELSE
*             oRegion := QRegion( oPolygon, 1 ) // Winding Fill ( default )
*          ENDIF
*          ::oQTObject:setMask( oRegion )
      ENDCASE
   ENDIF
   RETURN NIL


/*----------------------------------------------------------------------*/

METHOD ResetRegion( ) CLASS VIRTUALWINDOW
   ::oQTObject:clearMask( )
   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Release() CLASS VIRTUALWINDOW

   LOCAL i

   IF ::nType != WND_MAIN
      ::oEventLoop:Exit( 0 )
   ENDIF

   // Remove each control of the window
   FOR i := 1 TO len(::aControls)
      IF ! hb_IsNil( ::aControls[ i ] )
         ::aControls[ i ]:Release()  // it calls :close()
      ENDIF
   NEXT i

   Super:Release()  // it calls :close()

   ::oQTObject := NIL

*   IF ::nType == WND_MAIN
*      HMGAPP():Quit()
*   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/


